@@ -1,29 +1,60 @@
 ## -*- truncate-lines: t; -*-
-## Copyright (C) 2008-18  Enrico Schumann
+## Copyright (C) 2008-19  Enrico Schumann
 
 position <- function(amount, ...)
     UseMethod("position")
 
 position.default <- function(amount, timestamp, instrument,
                              when, drop.zero = FALSE,
-                             account = NULL, ...) {
-    
+                             account = NULL,
+                             use.names = NULL, ...) {
+
+    dim.amount <- dim(amount)
+    is.amount.matrix  <- !is.null(dim.amount) &&
+                         sum(dim.amount > 1L) == 2L
+    is.amount.matrix1 <- !is.null(dim.amount) &&
+                         sum(dim.amount > 1L) == 1L
+
     no.instruments <- FALSE ## are all instruments missing/NA?
+    if (((!.isFALSE(use.names) && missing(instrument)) ||
+            isTRUE(use.names))) {
+
+        if (is.amount.matrix1) {
+            nm <- colnames(amount)
+            amount <- c(amount)
+        } else if (is.amount.matrix) {
+            ## instrument is required
+            nm <- rep(colnames(amount), each = dim.amount[1L])
+            amount <- c(amount)
+        } else
+            nm <- names(amount)
+        if (!is.null(nm))
+            instrument <- nm
+    }
+
+
     if (missing(instrument) ||
         !length(instrument) ||
         all(is.na(instrument))) {
+        if (is.amount.matrix)
+            stop(sQuote("amount"), " is a matrix but ",
+                 sQuote("instrument"), " is missing")
         instrument <- rep.int("", length(amount))
         no.instruments <- TRUE
-    }
+    } else if (is.amount.matrix && length(instrument) == dim.amount[2L])
+        instrument <- rep(instrument, each = dim.amount[1L])
 
     no.timestamp <- FALSE
     if (missing(timestamp) ||
         !length(timestamp) ||
         all(is.na(timestamp))) {
-        timestamp <- rep(1, length(amount))
+        timestamp <- if (is.amount.matrix)
+                         seq_len(dim.amount[1L])
+                     else
+                         rep(1, length(amount))
         no.timestamp <- TRUE
     }
-
+    
     len <- max(length(amount),
                length(timestamp),
                length(instrument),
@@ -41,7 +72,7 @@ position.default <- function(amount, timestamp, instrument,
         if (no.timestamp)
             warning(sQuote("when"),
                     " specified, but no valid timestamp supplied")
-        if (is.character(when)) {            
+        if (is.character(when)) {
             if (when[[1L]] == "last" ||
                 when[[1L]] == "newest" ||
                 when[[1L]] == "latest")
@@ -50,6 +81,7 @@ position.default <- function(amount, timestamp, instrument,
                 when <- unique(timestamp)
             else if (when[[1L]] == "endofmonth" ||
                      when[[1L]] == "lastofmonth") {
+                ## TODO preferable?
                 ## when <- last(timestamp,
                 ##              format(as.Date(timestamp), "%Y-%m"))
                 timestamp <- as.Date(timestamp)
@@ -61,7 +93,7 @@ position.default <- function(amount, timestamp, instrument,
                 when <- last(timestamp,
                              format(timestamp, "%Y-%m-%d"))
             } else if (when[[1L]] == "first" ||
-                     when[1L] == "oldest")
+                       when[1L] == "oldest")
                 when <- min(timestamp)
         }
     }
@@ -82,14 +114,17 @@ position.default <- function(amount, timestamp, instrument,
     if (anyNA(timestamp) && !is.unsorted(timestamp, na.rm = TRUE))
         warning("timestamp has NA values")
 
-    if (!is.null(account) && !identical(account, FALSE))                      
+    if (!is.null(account) && !identical(account, FALSE))
         instrument <- paste(account, "%SEP%", instrument, sep = "")
-        
+
     nw <- length(when)
     nm <- sort(unique(instrument))
     pos <- array(0, dim = c(nw, length(nm)))
     colnames(pos) <- gsub("%SEP%", ".", nm, fixed = TRUE)
-    rownames(pos) <- if (no.timestamp) rep("", length(when)) else as.character(when)
+    rownames(pos) <- if (no.timestamp)
+                         rep("", length(when))
+                     else
+                         as.character(when)
     for (j in seq_len(nw)) {
         for (i in seq_along(nm)) {
             ri  <-  nm[i] == instrument
@@ -113,8 +148,8 @@ position.default <- function(amount, timestamp, instrument,
         nm[] <- NA
     attr(pos, "timestamp") <- if (no.timestamp) NA else when
     attr(pos, "instrument") <- gsub(".*%SEP%(.*?)", "\\1", nm)
-    if (!is.null(account))         
-        attr(pos, "account") <- gsub("(.*)%SEP%.*", "\\1", nm)    
+    if (!is.null(account))
+        attr(pos, "account") <- gsub("(.*)%SEP%.*", "\\1", nm)
     class(pos) <- "position"
     pos
 }
@@ -131,7 +166,9 @@ position.journal <- function(amount, when,
 
     position.default(amount, timestamp, instrument, when,
                      drop.zero = drop.zero,
-                     account = account, ...)
+                     account = account,
+                     use.names = FALSE,
+                     ...)
 }
 
 position.btest <- function(amount, when, ...,
@@ -159,7 +196,7 @@ position.btest <- function(amount, when, ...,
 }
 
 print.position <- function(x, ..., sep = ":") {
-    if (dim(x)[[2L]] == 0L) ## empty position
+    if (dim(x)[[2L]] == 0L)  ## empty position
         return(invisible(x))
     original.x <- x
     ## if (!is.na(sep))
@@ -178,19 +215,21 @@ print.position <- function(x, ..., sep = ":") {
             pos[all_i$position > 0] <- unclass(x)[all_i$position]
             x <- pos
         }
-    }        
+    }
     if (!all(is.na(instrument)))
         colnames(x) <- instrument
 
-    if (all(is.na(timestamp)) || (is.character(timestamp) && all(timestamp == "")))
+    if (all(is.na(timestamp)) ||
+        (is.character(timestamp) && all(timestamp == "")))
         rownames(x) <- NULL
-    if (all(is.na(instrument)) || (is.character(instrument) && all(instrument == "")))
+    if (all(is.na(instrument)) ||
+        (is.character(instrument) && all(instrument == "")))
         colnames(x) <- NULL
 
     attr(x, "account") <- NULL
     attr(x, "instrument") <- NULL
     attr(x, "timestamp") <- NULL
-    if (dim(x) == c(1L,1L) &&
+    if (all(dim(x) == c(1L, 1L)) &&
         is.null(rownames(x)) &&
         is.null(colnames(x))) {
         cat(x, "\n")
@@ -218,7 +257,7 @@ as.data.frame.position <- function(x, ...) {
     ans <- c(x)
     dim(ans) <- dim(x)
     ans <- as.data.frame(ans)
-    
+
     row.names(ans) <- as.character(attr(x, "timestamp"))
     names(ans) <- attr(x, "instrument")
     ans
@@ -267,22 +306,22 @@ as.zoo.position <- function(x, ...) {
     }
     s[is.na(s)] <- ""
     gs <- sort(unique(s))
-    
+
     list.gs <- strsplit(gs, sep, perl = perl)
 
     all.acc <- NULL
     for (i in seq_along(list.gs)) {
         if ((lg <- length(list.gs[[i]])) == 1L)
             next
-        
+
         tmp <- NULL
         for (j in seq_len(lg-1))
             tmp <- c(tmp, paste(list.gs[[i]][1:j],
                                 collapse = sep))
         all.acc <- c(all.acc, tmp)
-    }    
+    }
     all.acc <- sort(unique(c(all.acc, s)))
-    
+
     ## LEVEL
     level <- as.numeric(
         unlist(lapply(gregexpr(sep, all.acc),
@@ -300,23 +339,23 @@ acc.split <- function(account, sep, perl = FALSE, tree = FALSE) {
     .Deprecated(".expand")
     account[is.na(account)] <- ""
     gs <- sort(unique(account))
-    
+
     list.gs <- strsplit(gs, sep, perl = perl)
 
     all.acc <- NULL
     for (i in seq_along(list.gs)) {
         if ((lg <- length(list.gs[[i]])) == 1L)
             next
-        
+
         tmp <- NULL
         for (j in seq_len(lg-1))
             tmp <- c(tmp, paste(list.gs[[i]][1:j],
                                 collapse = sep))
         all.acc <- c(all.acc, tmp)
-    }    
+    }
     all.acc <- sort(unique(c(all.acc, account)))
     positions <- match(account, all.acc)
-    
+
     ## LEVEL
     level <- as.numeric(
         unlist(lapply(gregexpr(sep, all.acc),
@@ -332,14 +371,14 @@ acc.split <- function(account, sep, perl = FALSE, tree = FALSE) {
             gsub(paste0(".*", sep, "([^", substr(sep,1,1), "]+)$"),
                  "\\1", x, perl = TRUE)
         }
-        
+
         sp <- spaces(4*(level - 1))
-        tree1 <- paste0(sp, leaf(all.acc)) 
+        tree1 <- paste0(sp, leaf(all.acc))
 
         tree2 <- paste0(.tree(level), leaf(all.acc))
         tree3 <- paste0(.tree(level, TRUE), leaf(all.acc))
         tree3 <- enc2utf8(tree3)
-        
+
         res <- data.frame(account = all.acc,
                           level,
                           tree_indent  = tree1,
@@ -351,10 +390,10 @@ acc.split <- function(account, sep, perl = FALSE, tree = FALSE) {
     }
     attr(res, "positions") <- positions
     res
-} 
+}
 
 .leaf <- function(x, sep, perl = TRUE) {
-    ## return last level 
+    ## return last level
     ## level::sublevel::...::subbest_sublevel
     ## => the regexp used greedy matching
     gsub(paste0(".*", sep, "(.*)"), "\\1", x, perl = TRUE)
@@ -382,10 +421,10 @@ acc.split <- function(account, sep, perl = FALSE, tree = FALSE) {
                             lv == i+1))
                     group.end[g] <- max(which(next_l))
             }
-            
+
             group.start <- group.start[-length(group.start)]
             group.end <- group.end[-length(group.end)]
-            
+
             for (g in 1:length(group.start)) {
                 if (group.start[g] == group.end[g])
                     next
@@ -407,5 +446,5 @@ acc.split <- function(account, sep, perl = FALSE, tree = FALSE) {
             indent <- gsub("|  ", "\u2502  ", indent, fixed = TRUE)
         }
         indent
-    } 
+    }
 }
